-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    bdu_fsm.vhd
-- Description:  Controll bdu working and getting data from bus
--               Send values when robot move one position
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
--use ieee.numeric_std.all;
--use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity BDU_FSM_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- INPUT/OUTPUT DATA --------------------- 
      S_0            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_1            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_2            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      S_3            : in std_logic_vector(DATA_WIDTH-1 downto 0);
      NOT_CYC_WRITE  : in std_logic;
      BAR            : out std_logic_vector(DATA_WIDTH-1 downto 0);
      -- ----------------- BDU INTERFACE -------------------------- 
      S_0_BDU        : out std_logic_vector(DATA_WIDTH-1 downto 0);
      S_1_BDU        : out std_logic_vector(DATA_WIDTH-1 downto 0);
      S_2_BDU        : out std_logic_vector(DATA_WIDTH-1 downto 0); 
      S_3_BDU        : out std_logic_vector(DATA_WIDTH-1 downto 0);   
      BAR_BDU        : in std_logic_vector(DATA_WIDTH-1 downto 0);
      ACT_DBU        : out std_logic;
      VLD_BDU        : in  std_logic
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture BDU_FSM_ARCH of BDU_FSM_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, get_value_state, evaluate_state); --stavy FSM

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- data signals
signal reg_S_0                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_S_1                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_S_2                : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_S_3                : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_we_s               : std_logic;
signal sig_act                : std_logic;
signal sig_vld                : std_logic;

signal sig_change_val         : std_logic;

begin
   -- detect event on input data => enable evaluating
   change_register : process (CLK, reg_S_0, reg_S_1, reg_S_2, reg_S_3, S_0, S_1, S_2, S_3, sig_vld, sig_change_val)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_change_val <= '0';
         elsif (reg_S_0 /= S_0 or reg_S_1 /= S_1 or reg_S_2 /= S_2 or reg_S_3 /= S_3) then
            sig_change_val <= '1';
         elsif (sig_vld = '1') then
            sig_change_val <= '0';
         end if;
      end if;
   end process;
   
   -- write to input registers A,B,C
   input_registers : process (sig_we_s, CLK, RST, s_0, S_1, S_2, S_3)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            reg_S_0 <= (others => '0');
            reg_S_1 <= (others => '0');
            reg_S_2 <= (others => '0');
            reg_S_3 <= (others => '0');
         elsif (sig_we_s = '1') then
            reg_S_0 <= S_0;
            reg_S_1 <= S_1;
            reg_S_2 <= S_2;
            reg_S_3 <= S_3;
         end if;
      end if;
   end process;

   S_0_BDU     <= reg_S_0;
   S_1_BDU     <= reg_S_1;
   S_2_BDU     <= reg_S_2;
   S_3_BDU     <= reg_S_3;
              
   BAR <= BAR_BDU;
      
      -- control interface
   ACT_DBU     <= sig_act;
   sig_vld     <= VLD_BDU;
   
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic : process (state_reg, sig_vld, NOT_CYC_WRITE, sig_change_val)
   begin
     state_next <= state_reg;  
     
     case state_reg is
        when init_state =>
          if (NOT_CYC_WRITE = '1' and sig_change_val = '1') then 
             state_next <= get_value_state;
             sig_we_s <= '1';
             sig_act <= '0';
          else 
             state_next <= init_state;
             sig_we_s <= '0';
             sig_act <= '0';
          end if;                
        
        when get_value_state => 
          state_next <= evaluate_state;
          sig_we_s <= '0';
          sig_act <= '1';
          
        when evaluate_state => 
          if (sig_vld = '1') then 
             state_next <= init_state;
             sig_we_s <= '0';
             sig_act <= '1';
          else 
             state_next <= evaluate_state;
             sig_we_s <= '0';
             sig_act <= '1';
          end if;
      end case;      
   end process;  
end architecture;
