-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    move.vhd
-- Description:  Controll robot moving, send speed to output and count time
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity MOVE_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16     
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      
      ACT            : in std_logic;
      VLD            : out std_logic;

      -- ----------------- DIRECION INPUT -------------------------
      L              : in std_logic;
      R              : in std_logic;
      T              : in std_logic;
      B              : in std_logic;
      -- ----------------- SPEED OUTPUT ---------------------------
      X_SPEED        : out std_logic_vector(DATA_WIDTH - 1 downto 0);
      Y_SPEED        : out std_logic_vector(DATA_WIDTH - 1 downto 0)
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture MOVE_ARCH of MOVE_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, count_state, handshake_state); 

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
--constant MAX_COUNT       : std_logic_vector(31 downto 0):= conv_std_logic_vector(10, 32); -- movement time = t/T
constant MAX_COUNT       : std_logic_vector(31 downto 0):= conv_std_logic_vector(216000000, 32); -- movement time = t/T
constant SPEED           : std_logic_vector(DATA_WIDTH - 1 downto 0):= conv_std_logic_vector(1, DATA_WIDTH); -- speed BOX_SIZE/t
-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- signals
signal sig_en              : std_logic;
signal sig_reset           : std_logic;
signal sig_out             : std_logic;

signal sig_count           : std_logic_vector(31 downto 0);

signal sig_x_speed         : std_logic_vector(DATA_WIDTH - 1 downto 0);
signal sig_y_speed         : std_logic_vector(DATA_WIDTH - 1 downto 0);



begin
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic: process( state_reg, sig_count, ACT, L, R, T, B) --stavova logika
   begin
      case state_reg is
         when init_state => --nic se nedeje
            if  (ACT = '1' and L = '0' and R = '0' and T='0' and B='0') then
               VLD         <= '0';
               sig_en      <= '0';
               sig_reset   <= '0';
               sig_out     <= '0';
               state_next  <= handshake_state;
            elsif (ACT = '1') then
               VLD         <= '0';
               sig_en      <= '1';
               sig_reset   <= '0';
               sig_out     <= '1';
               state_next  <= count_state;
            else
               VLD         <= '0';
               sig_en      <= '0';
               sig_reset   <= '1';
               sig_out     <= '0';
               state_next  <= init_state;
            end if;
         when count_state =>
            if (sig_count < MAX_COUNT) then
               VLD         <= '0';
               sig_en      <= '1';
               sig_reset   <= '0';
               sig_out     <= '1';
               state_next  <= count_state;
            else
               VLD         <= '1';
               sig_en      <= '0';
               sig_reset   <= '0';
               sig_out     <= '0';
               state_next  <= handshake_state;
            end if;
         when handshake_state =>
            if (ACT = '1') then
               VLD         <= '1';
               sig_en      <= '0';
               sig_reset   <= '1';
               sig_out     <= '0';
               state_next  <= handshake_state;
            else
               VLD         <= '0';
               sig_en      <= '0';
               sig_reset   <= '1';
               sig_out     <= '0';
               state_next  <= init_state;
            end if;
         when others => 
            VLD         <= '0';
            sig_en      <= '0';
            sig_reset   <= '1';
            sig_out     <= '0';
            state_next  <= init_state;
      end case;
   end process;  
 
   -- count nubers of send values
   counter_r : process (CLK, sig_reset, sig_en)
   variable COUNT : std_logic_vector(31 downto 0):= (OTHERS => '0');
   begin
      if (sig_reset = '1') then
         COUNT := (OTHERS => '0');
      elsif (CLK='1' and CLK'event) then
         if (sig_en = '1') then
            if COUNT = MAX_COUNT then
               COUNT := (OTHERS => '0');
            else
               COUNT := COUNT + 1; 
            end if;
         end if;
      end if; 
      sig_count <= COUNT;
   end process;
   
   
   -- multiplexor to set output data
   input_mux : process(L, R, T, B)
   variable S : std_logic_vector(3 downto 0);
   begin
      S := L & R & T & B;
      case S is
         when "1000"  => sig_x_speed <= - SPEED; 
                         sig_y_speed <= (OTHERS => '0');
         when "0100"  => sig_x_speed <= SPEED; 
                         sig_y_speed <= (OTHERS => '0');
         when "0010"  => sig_x_speed <= (OTHERS => '0');
                         sig_y_speed <= SPEED;
         when "0001"  => sig_x_speed <= (OTHERS => '0');
                         sig_y_speed <= - SPEED; 
         when others  => sig_x_speed <= (OTHERS => '0');
                         sig_y_speed <= (OTHERS => '0');
      end case;  
   end process;
   
   -- multiplexor to set output data
   output_mux : process(sig_out, sig_x_speed, sig_y_speed)
   begin
      case sig_out is
         when '1'    => X_SPEED <= sig_x_speed; 
                        Y_SPEED <= sig_y_speed;
         when '0'    => X_SPEED <= (OTHERS => '0'); 
                        Y_SPEED <= (OTHERS => '0');
         when others => X_SPEED <= (OTHERS => '0'); 
                        Y_SPEED <= (OTHERS => '0');
      end case;  
   end process;
   
end architecture;
