-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    fifo.vhd
-- Description:  Fifo memory to save positions list from pfu 
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity FIFO_ENT is
   generic
   (
      SIZE           : integer := 15;
      DATA_WIDTH     : integer := 16     
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      -- ----------------- FIFO INTERFACE --------------------------------
      DATA_IN        : in std_logic_vector (2*DATA_WIDTH-1 downto 0);  
      DATA_OUT       : out std_logic_vector(2*DATA_WIDTH-1 downto 0);
      READ_EN        : in std_logic;  
      WRITE_EN       : in std_logic; 
      EMPTY          : out std_logic;  
      FULL           : out std_logic;
      ERROR          : out std_logic
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture FIFO_ENT_ARCH of FIFO_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type memory_type is array (0 to SIZE-1) of std_logic_vector(2*DATA_WIDTH-1 downto 0);
-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- signals 
signal memory : memory_type :=(others => (others => '0')); 

begin
   fifo : process(CLK,RST)
   variable read_ptr, write_ptr : std_logic_vector(3 downto 0) :=(others => '0');  -- READ_EN and write pointers
   variable isempty , isfull : std_logic :='0';
   begin
      if ( RST = '1' ) then
         memory <= (others => (others => '0'));
         --memory(0)  <= std_logic_vector(to_unsigned(1, DATA_WIDTH)) & std_logic_vector(to_unsigned(6, DATA_WIDTH));
         --memory(1)  <= std_logic_vector(to_unsigned(1, DATA_WIDTH)) & std_logic_vector(to_unsigned(5, DATA_WIDTH));
         --memory(2)  <= std_logic_vector(to_unsigned(2, DATA_WIDTH)) & std_logic_vector(to_unsigned(5, DATA_WIDTH));
         --memory(3)  <= std_logic_vector(to_unsigned(3, DATA_WIDTH)) & std_logic_vector(to_unsigned(5, DATA_WIDTH));
         --memory(4)  <= std_logic_vector(to_unsigned(4, DATA_WIDTH)) & std_logic_vector(to_unsigned(5, DATA_WIDTH));
         --memory(5)  <= std_logic_vector(to_unsigned(4, DATA_WIDTH)) & std_logic_vector(to_unsigned(6, DATA_WIDTH));
         --memory(6)  <= std_logic_vector(to_unsigned(5, DATA_WIDTH)) & std_logic_vector(to_unsigned(6, DATA_WIDTH));
         --memory(7)  <= std_logic_vector(to_unsigned(6, DATA_WIDTH)) & std_logic_vector(to_unsigned(6, DATA_WIDTH));
         --memory(8)  <= std_logic_vector(to_unsigned(6, DATA_WIDTH)) & std_logic_vector(to_unsigned(5, DATA_WIDTH));
         --memory(9)  <= std_logic_vector(to_unsigned(6, DATA_WIDTH)) & std_logic_vector(to_unsigned(4, DATA_WIDTH));
         --memory(10) <= std_logic_vector(to_unsigned(6, DATA_WIDTH)) & std_logic_vector(to_unsigned(3, DATA_WIDTH));
         --memory(11) <= std_logic_vector(to_unsigned(6, DATA_WIDTH)) & std_logic_vector(to_unsigned(2, DATA_WIDTH));
         --memory(12) <= std_logic_vector(to_unsigned(6, DATA_WIDTH)) & std_logic_vector(to_unsigned(1, DATA_WIDTH));
         EMPTY <= '1';
         --EMPTY <= '0';
         FULL <= '0';
         DATA_OUT <= (others => '0');
         read_ptr := (others => '0');
         write_ptr := (others => '0');
         --write_ptr := "1101";--(others => '0');
		 --write_ptr := "0011";--(others => '0');
         isempty := '1';
         --isempty := '0';
         isfull := '0';
         ERROR <= '0';
      elsif ( rising_edge( CLK ) ) then
         if ( READ_EN = '0' and WRITE_EN = '0' ) then
            ERROR <= '0';
         end if;
         
         -- read value
         if ( READ_EN = '1' ) then
            -- read from empty fifo
            if ( isempty = '1' ) then
               ERROR <= '1'; 
            else
               DATA_OUT <= memory(conv_integer(read_ptr));
               isfull := '0';
               FULL <= '0';
               ERROR <= '0';
               -- increment read pointer
               if ( read_ptr = conv_std_logic_vector(SIZE-1,4)) then
                  read_ptr := (others => '0');
               else
                  read_ptr := read_ptr + '1';
               end if; 
               -- fifo is empty after read value
               if ( read_ptr = write_ptr ) then
                  isempty  := '1';
                  EMPTY <= '1';
               end if;
            end if;
         end if;
         
         --write walue
         if ( WRITE_EN = '1' ) then
            -- write to full fifo
            if ( isfull = '1' ) then
               ERROR <= '1';
            else
               memory(conv_integer(write_ptr)) <= DATA_IN;
               ERROR <= '0';
               isempty := '0';
               EMPTY <= '0';
               -- increment write pointer
               if ( write_ptr = conv_std_logic_vector(SIZE-1,4) ) then
                  write_ptr := (others => '0');
               else
                  write_ptr := write_ptr + '1';
               end if;
               -- fifo if full after write value
               if ( write_ptr = read_ptr ) then
                  isfull := '1';
                  FULL <= '1';
               end if;
            end if;
         end if;
      end if;
   end process;   
end architecture;