-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    mc.vhd
-- Description:  Memory Controller
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.1.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity MC_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16;  
      ADDR_WIDTH  : integer := 13       
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic; 
      
      -- ----------------- MEMORY INTERFACE --------------------------------- 
      ADDR           : out std_logic_vector(ADDR_WIDTH-1 downto 0);
      WE             : out std_logic;
      DI             : out std_logic_vector(3 downto 0);
      DO             : in std_logic_vector(3 downto 0);
      EN             : out  std_logic;
      SSR            : out std_logic; 
      MCLK           : out std_logic;        
              
      -- ----------------- INPUT/OUTPUT INTERFACE ---------------------------
      ADDR_X         : in std_logic_vector(DATA_WIDTH-1 downto 0);
      ADDR_Y         : in std_logic_vector(DATA_WIDTH-1 downto 0);
      RD_WR          : in std_logic;
      DATA_OUT       : out std_logic_vector(3 downto 0);
      DATA_IN        : in std_logic_vector(3 downto 0);
      ACT            : in std_logic;
      VLD            : out std_logic
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture MC_ARCH of MC_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state is (init_state, mem_read_state, mem_write_state); --stavy FSM

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
constant MAX_SIZE   : integer := 89;
-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals

-- control signals
signal sig_vld                : std_logic;
signal state_reg, state_next  : state; --aktualni a budouci stav FSM
signal sig_addr               : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_mult               : std_logic_vector(2*DATA_WIDTH-1 downto 0);

begin
   ADDR     <= sig_addr(ADDR_WIDTH-1 downto 0);
   WE       <= RD_WR;
   DI       <= DATA_IN;
   DATA_OUT <= DO;
   EN       <= ACT;
   SSR      <= RST;
   MCLK     <= CLK;
   
   -- mult
   mult : process(ADDR_Y)
   begin
      sig_mult <= ADDR_Y * std_logic_vector(to_unsigned(MAX_SIZE, DATA_WIDTH));
      --sig_addr <= ADDR_X + sig_mult(DATA_WIDTH-1 downto 0);
   end process;
   
   -- add
   add : process(ADDR_X, sig_mult)
   begin
      sig_addr <= ADDR_X + sig_mult(DATA_WIDTH-1 downto 0);
   end process;
   
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic : process (state_reg, ACT, RD_WR)
   begin
     state_next <= state_reg;  
     
     case state_reg is
        when init_state =>
          if (ACT = '1' and RD_WR = '0') then 
             state_next <= mem_read_state;
             sig_vld <= '0';
          elsif (ACT = '1' and RD_WR = '1') then 
             state_next <= mem_write_state;
             sig_vld <= '0';
          else 
             state_next <= init_state;
             sig_vld <= '0';
          end if;                
        
        when mem_read_state => 
          if (ACT = '1') then 
             state_next <= mem_read_state;
             sig_vld <= '1';
          else 
             state_next <= init_state; 
             sig_vld <= '0';
          end if;
          
        when mem_write_state => 
          if (ACT = '1') then 
             state_next <= mem_write_state;
             sig_vld <= '1';
          else 
             state_next <= init_state; 
             sig_vld <= '0';
          end if;
      end case;      
   end process;  

   VLD <= sig_vld;   
end architecture;
