-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    mmu.vhd
-- Description:  Map Memory Unit
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.9.2012 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity MMU_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16;  
      ADDR_WIDTH  : integer := 13       
   );
   
   port
   (
      -- ----------------- PORT A ---------------------------
      CLK_A             : in  std_logic;
      RST_A             : in  std_logic;        
      ADDR_X_A          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      ADDR_Y_A          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      RD_WR_A           : in std_logic;
      DATA_OUT_A        : out std_logic_vector(3 downto 0);
      DATA_IN_A         : in std_logic_vector(3 downto 0);
      ACT_A             : in  std_logic;
      VLD_A             : out std_logic;
      
      -- ----------------- PORT B ---------------------------
      CLK_B             : in  std_logic;
      RST_B             : in  std_logic;        
      ADDR_X_B          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      ADDR_Y_B          : in std_logic_vector(DATA_WIDTH-1 downto 0);
      RD_WR_B           : in std_logic;
      DATA_OUT_B        : out std_logic_vector(3 downto 0);
      DATA_IN_B         : in std_logic_vector(3 downto 0);
      ACT_B             : in  std_logic;
      VLD_B             : out std_logic
 
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture MMU_ARCH of MMU_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
                
-- control signals
signal sig_addra           : std_logic_vector(ADDR_WIDTH-1 downto 0);
signal sig_wea             : std_logic;
signal sig_dia             : std_logic_vector(3 downto 0);
signal sig_doa             : std_logic_vector(3 downto 0);
signal sig_ena             : std_logic;
signal sig_ssra            : std_logic; 
signal sig_clka            : std_logic; 

signal sig_addrb           : std_logic_vector(ADDR_WIDTH-1 downto 0);
signal sig_web             : std_logic;
signal sig_dib             : std_logic_vector(3 downto 0);
signal sig_dob             : std_logic_vector(3 downto 0);
signal sig_enb             : std_logic;
signal sig_ssrb            : std_logic; 
signal sig_clkb            : std_logic; 

signal sig_act_b           : std_logic;


begin

   -- when adres of port A is the same as port B, then port B must wait
   synch : process(ACT_A,ACT_B, ADDR_X_A, ADDR_Y_A, ADDR_X_B, ADDR_Y_B)
   begin
      if (ADDR_X_A = ADDR_X_B and ADDR_Y_A = ADDR_Y_B) then
         sig_act_b <= ACT_B and not ACT_A;
      else
         sig_act_b <= ACT_B;
      end if;
   end process; 

   mca : entity work.MC_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH,
      ADDR_WIDTH  => ADDR_WIDTH
   )   
   port map(
      CLK            => CLK_A,
      RST            => RST_A,
      
      -- ----------MEMORY INTERFACE ----------
      ADDR           => sig_addra,  
      WE             => sig_wea,
      DI             => sig_dia,
      DO             => sig_doa,
      EN             => sig_ena,
      SSR            => sig_ssra, 
      MCLK           => sig_clka,       
              
      -- ----------INPUT/OUTPUT INTERFACE ----
      ADDR_X         => ADDR_X_A,
      ADDR_Y         => ADDR_Y_A,
      RD_WR          => RD_WR_A,
      DATA_OUT       => DATA_OUT_A,
      DATA_IN        => DATA_IN_A,
      ACT            => ACT_A,
      VLD            => VLD_A
   );
   
   mcb : entity work.MC_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH,
      ADDR_WIDTH  => ADDR_WIDTH
   )   
   port map(
      CLK            => CLK_B,
      RST            => RST_B,
      
      -- ----------MEMORY INTERFACE ----------
      ADDR           => sig_addrb,  
      WE             => sig_web,
      DI             => sig_dib,
      DO             => sig_dob,
      EN             => sig_enb,
      SSR            => sig_ssrb, 
      MCLK           => sig_clkb,       
              
      -- ----------INPUT/OUTPUT INTERFACE ----
      ADDR_X         => ADDR_X_B,
      ADDR_Y         => ADDR_Y_B,
      RD_WR          => RD_WR_B,
      DATA_OUT       => DATA_OUT_B,
      DATA_IN        => DATA_IN_B,
      ACT            => sig_act_b,
      VLD            => VLD_B
   );
   
   bram : entity work.BRAM_ENT
   generic map(
      DATA_WIDTH  => 4,
      ADDR_WIDTH  => ADDR_WIDTH
   )   
   port map(
      -- ---------- PORT A ----------
      ADDRA          => sig_addra,  
      WEA            => sig_wea,
      DIA            => sig_dia,
      DOA            => sig_doa,
      ENA            => sig_ena,
      SSRA           => sig_ssra, 
      CLKA           => sig_clka,       
              
      -- ---------- PORT A ----------
      ADDRB          => sig_addrb,  
      WEB            => sig_web,
      DIB            => sig_dib,
      DOB            => sig_dob,
      ENB            => sig_enb,
      SSRB           => sig_ssrb, 
      CLKB           => sig_clkb
   );
  
end architecture;
