-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    mu.vhd
-- Description:  Map Unit
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.9.2012 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity MU_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic;
      ACT            : in  std_logic;
      -- ----------------- INPUT/OUTPUT INTERFACE ----------------------------------
      X              : in  std_logic_vector(DATA_WIDTH-1 downto 0);  
      Y              : in  std_logic_vector(DATA_WIDTH-1 downto 0); 
      BARRIER        : in  std_logic_vector(3 downto 0);
      VLD            : out std_logic;
      -- ----------------- MEMORY INTERFACE ---------------------------------   
      ADDR_X         : out std_logic_vector(DATA_WIDTH-1 downto 0);
      ADDR_Y         : out std_logic_vector(DATA_WIDTH-1 downto 0);
      RD_WR          : out std_logic;
      DATA_OUT       : out std_logic_vector(3 downto 0);
      DATA_IN        : in std_logic_vector(3 downto 0);
      MEM_ACT        : out std_logic;
      MEM_VLD        : in std_logic   
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture MU_ARCH of MU_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
signal sig_count              : std_logic_vector(2 downto 0);
                
-- control signals
signal sig_vld                : std_logic;
signal sig_act                : std_logic;
signal sig_next               : std_logic;

-- muu signals
signal sig_addr_x             : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_addr_y             : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_valu               : std_logic;
signal sig_mem_act            : std_logic;
signal sig_mem_vld            : std_logic;  

signal sig_count_rst          : std_logic;

begin
   sig_count_rst <= RST or not(ACT);

   -- counter 0 - 5 for controlling mux
   counter : process (sig_count_rst, sig_next, CLK)
   variable COUNT_INT: integer;
   begin
      if( rising_edge( CLK ) ) then 
         if (sig_count_rst = '1') then
            COUNT_INT := 0;
         elsif ( sig_next='1' ) then
            if (COUNT_INT = 5) then
               COUNT_INT := 0;
            else
               COUNT_INT := COUNT_INT + 1; 
            end if;
         end if; 
      end if;
      sig_count <= conv_std_logic_vector(COUNT_INT, 3);
   end process;
   
   -- mux controlled by counter
   mux: process(X,Y,BARRIER,sig_count)
   begin
      sig_addr_x <= X;
      sig_addr_y <= Y;
      sig_valu <= '0';
      sig_act <= '0';
      sig_vld <= '0';
      
      case sig_count is
         -- my position
         when "000" =>  sig_addr_x <= X;
                        sig_addr_y <= Y;
                        sig_valu <= '0';
                        sig_act <= '1';
                        sig_vld <= '0';
         -- left
         when "001" =>  sig_addr_x <= X - 1;
                        sig_addr_y <= Y;
                        sig_valu <= BARRIER(0);
                        sig_act <= '1';
                        sig_vld <= '0';
         -- right
         when "010" =>  sig_addr_x <= X + 1;
                        sig_addr_y <= Y;
                        sig_valu <= BARRIER(1);
                        sig_act <= '1';
                        sig_vld <= '0';
         -- down
         when "011" =>  sig_addr_x <= X;
                        sig_addr_y <= Y + 1;
                        sig_valu <= BARRIER(2);
                        sig_act <= '1';
                        sig_vld <= '0';
         -- up
         when "100" =>  sig_addr_x <= X;
                        sig_addr_y <= Y - 1;
                        sig_valu <= BARRIER(3);
                        sig_act <= '1';
                        sig_vld <= '0';
         -- idle
         when others => sig_addr_x <= X;
                        sig_addr_y <= Y;
                        sig_valu <= '0';
                        sig_act <= '0';
                        sig_vld <= '1';
      end case;
   end process;
   
   VLD <= sig_vld and ACT;
   
   -- generate delay to controll counter
   delay : process(sig_mem_vld, CLK)
   begin
      if( rising_edge(CLK) ) then   
         if (sig_mem_vld = '1') then
            sig_next <= '1';
         else
            sig_next <= '0';
         end if;
      end if;
   end process;

   
   sig_mem_act <= sig_act and not(sig_next) and ACT;
   
   muu : entity work.MUU_ENT
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      CLK           => CLK,
      RST           => RST,
      
      -- input/output interface
      ACT           => sig_mem_act, 
      VLD           => sig_mem_vld, 
      VALU          => sig_valu, 
      X             => sig_addr_x, 
      Y             => sig_addr_y, 
           
      -- memory interface      
      ADDR_X        => ADDR_X,
      ADDR_Y        => ADDR_Y,
      RD_WR         => RD_WR,
      DATA_OUT      => DATA_OUT,
      DATA_IN       => DATA_IN,
      MEM_ACT       => MEM_ACT,
      MEM_VLD       => MEM_VLD
   );
  
end architecture;
