-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    peu.vhd
-- Description:  Position Evaluate Unit
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         28.9.2012 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity PEU_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic;
      ACT            : in  std_logic;

      -- ----------------- INPUT INTERFACE ----------------------------------
      DIST_A         : in  std_logic_vector(DATA_WIDTH-1 downto 0);  
      DIST_B         : in  std_logic_vector(DATA_WIDTH-1 downto 0); 
      DIST_C         : in  std_logic_vector(DATA_WIDTH-1 downto 0); 

     
      -- ----------------- OUTPUT INTERFACE ---------------------------------      
      X              : out std_logic_vector(DATA_WIDTH-1 downto 0);
      Y              : out std_logic_vector(DATA_WIDTH-1 downto 0);
      VLD            : out std_logic
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture PEU_ARCH of PEU_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
constant X_MAX          : integer := 4096;
constant EXP_X_MAX      : integer := 6;
constant Y_MAX          : integer := 4096;
constant EXP_Y_MAX      : integer := 6;

constant EXP_BOX_SIZE   : integer := 3;

-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- data signals
signal sig_dist_A2          : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_dist_B2          : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_dist_C2          : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_sub_B_A          : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_sub_B_C          : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_add_X_MAX        : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_add_Y_MAX        : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_app_X            : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_app_Y            : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_X                : std_logic_vector(2*DATA_WIDTH-1 downto 0);
signal sig_Y                : std_logic_vector(2*DATA_WIDTH-1 downto 0);
-- control signals
signal sig_mult1_vld        : std_logic;
signal sig_mult2_vld        : std_logic;
signal sig_mult3_vld        : std_logic;
signal sig_mult1_2_vld      : std_logic;
signal sig_mult2_3_vld      : std_logic;
signal sig_srl1_vld         : std_logic;
signal sig_srl2_vld         : std_logic;
signal sig_srl3_vld         : std_logic;
signal sig_srl4_vld         : std_logic;
signal sig_sub1_vld         : std_logic;
signal sig_sub2_vld         : std_logic;
signal sig_add1_vld         : std_logic;
signal sig_add2_vld         : std_logic;
signal sig_vld              : std_logic;

begin
   -- reset and write to output
   reset : process(RST,sig_Y,sig_X,sig_srl3_vld, sig_srl4_vld, CLK, ACT)
   begin
      if (RST = '1') then
         sig_vld <= '0';
         Y <= (others => '1'); 
         X <= (others => '1'); 
      elsif (sig_srl3_vld = '1' and sig_srl4_vld = '1' and ACT = '1') then
         if (CLK = '1' and CLK'event) then
            -- get DATA_WITDT bits to output
            sig_vld <= '1';
            Y <= sig_Y(DATA_WIDTH-1 downto 0);
            X <= sig_X(DATA_WIDTH-1 downto 0);
         end if;
      else
         sig_vld <= '0';
      end if;
   end process;

   -- sqr of input distances  
   mult1 : entity work.mult
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT_A       => DIST_A, 
      INPUT_B       => DIST_A, 
      START         => ACT, 
           
      -- output interface      
      MULT_RESULT   => sig_dist_A2,
      MULT_VLD      => sig_mult1_vld
   );
    
   mult2 : entity work.mult
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT_A       => DIST_B, 
      INPUT_B       => DIST_B, 
      START         => ACT, 
           
      -- output interface      
      MULT_RESULT   => sig_dist_B2,
      MULT_VLD      => sig_mult2_vld
   );

   mult3 : entity work.mult
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT_A       => DIST_C, 
      INPUT_B       => DIST_C, 
      START         => ACT, 
           
      -- output interface      
      MULT_RESULT   => sig_dist_C2,
      MULT_VLD      => sig_mult3_vld
   );
    
   sig_mult1_2_vld <= sig_mult1_vld and sig_mult2_vld;
   sig_mult2_3_vld <= sig_mult2_vld and sig_mult3_vld;
   
   -- sub
   sub1 : entity work.sub
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT_A       => sig_dist_B2, 
      INPUT_B       => sig_dist_A2, 
      START         => sig_mult1_2_vld, 
           
      -- output interface      
      SUB_RESULT   => sig_sub_B_A,
      SUB_VLD      => sig_sub1_vld
   );
    
   sub2 : entity work.sub
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT_A       => sig_dist_B2, 
      INPUT_B       => sig_dist_C2, 
      START         => sig_mult2_3_vld, 
           
      -- output interface      
      SUB_RESULT   => sig_sub_B_C,
      SUB_VLD      => sig_sub2_vld
   );   
   
   add1 : entity work.add
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT_A       => sig_sub_B_A, 
      INPUT_B       => std_logic_vector(to_unsigned(Y_MAX, 2*DATA_WIDTH)),
      START         => sig_sub1_vld, 
           
      -- output interface      
      ADD_RESULT    => sig_add_Y_MAX,
      ADD_VLD       => sig_add1_vld
   ); 
    
   add2 : entity work.add
   generic map(
      DATA_WIDTH  => DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT_A       => sig_sub_B_C, 
      INPUT_B       => std_logic_vector(to_unsigned(X_MAX, 2*DATA_WIDTH)),
      START         => sig_sub2_vld, 
           
      -- output interface      
      ADD_RESULT    => sig_add_X_MAX,
      ADD_VLD       => sig_add2_vld
   );
  
   
   -- shift right 
   srl1 : entity work.shiftr
   generic map(
      DATA_WIDTH  => 2*DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT         => sig_add_Y_MAX, 
      N             => EXP_Y_MAX+1, 
      START         => sig_add1_vld, 
           
      -- output interface      
      SHIFTR_RESULT   => sig_app_Y,
      SHIFTR_VLD      => sig_srl1_vld
   );

   srl2 : entity work.shiftr
   generic map(
      DATA_WIDTH  => 2*DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT         => sig_add_X_MAX, 
      N             => EXP_X_MAX+1, 
      START         => sig_add2_vld, 
          
      -- output interface      
      SHIFTR_RESULT   => sig_app_X,
      SHIFTR_VLD      => sig_srl2_vld
   );
   
   srl3 : entity work.shiftr
   generic map(
      DATA_WIDTH  => 2*DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT         => sig_app_Y, 
      N             => EXP_BOX_SIZE, 
      START         => sig_srl1_vld, 
          
      -- output interface      
      SHIFTR_RESULT   => sig_Y,
      SHIFTR_VLD      => sig_srl3_vld
   );
    
   srl4 : entity work.shiftr
   generic map(
      DATA_WIDTH  => 2*DATA_WIDTH
   )   
   port map(
      --CLK           => CLK,
      --RST           => RST,
      
      -- input interface
      INPUT         => sig_app_X, 
      N             => EXP_BOX_SIZE, 
      START         => sig_srl2_vld, 
           
      -- output interface      
      SHIFTR_RESULT   => sig_X,
      SHIFTR_VLD      => sig_srl4_vld
   );
   
   -- signal sig_vld to output
   VLD <= sig_vld;
end architecture;
