-----------------------------------------------------------------------------
-- Project Name: ROBOT
-- File Name:    pfu.vhd
-- Description:  Path Finding Unit
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         26.3.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;
-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity PFU_ENT is
   generic
   (
      DATA_WIDTH  : integer := 16      
   );
   
   port
   (
      CLK            : in  std_logic;
      RST            : in  std_logic;
	  RDY            : in  std_logic;
      
      -- ----------------- PFU_WB INTERFACE --------------------- 
      X_WB           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      Y_WB           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      ACT_WB         : out std_logic;
      VLD_WB         : in  std_logic;
      
      -- ----------------- ECU INTERFACE ------------------------ 
      X_ECU          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      Y_ECU          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      ACT_ECU        : out std_logic;
      VLD_ECU        : in  std_logic;
      
      -- ----------------- MMU INTERFACE ------------------------ 
      X_MMU          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      Y_MMU          : out std_logic_vector(DATA_WIDTH-1 downto 0);
      ACT_MMU        : out std_logic;
      VLD_MMU        : in  std_logic;
      RD_WR_MMU      : out std_logic;
      DATA_IN_MMU    : in std_logic_vector(3 downto 0)
   );   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture PFU_ARCH of PFU_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state, wait_state, read_pos_state, add_first_state, load_out_state, read_mem_state, detect_bar_state, go_to_state, goal_state); -- FSM states

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
-- target
constant X_TARGET       : std_logic_vector(DATA_WIDTH-1 downto 0):= conv_std_logic_vector(6, DATA_WIDTH);
constant Y_TARGET       : std_logic_vector(DATA_WIDTH-1 downto 0):= conv_std_logic_vector(1, DATA_WIDTH);
-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next: state_type;

-- data signals
signal sig_X                  : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_Y                  : std_logic_vector(DATA_WIDTH-1 downto 0);

signal sig_X_in               : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_Y_in               : std_logic_vector(DATA_WIDTH-1 downto 0);

signal sig_X_new              : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_Y_new              : std_logic_vector(DATA_WIDTH-1 downto 0);

signal sig_X_old              : std_logic_vector(DATA_WIDTH-1 downto 0);
signal sig_Y_old              : std_logic_vector(DATA_WIDTH-1 downto 0);

-- control signals
signal sig_dir                : std_logic_vector(1 downto 0);

signal sig_xy_sel             : std_logic;
signal sig_xy_we              : std_logic;
signal sig_xy_old_we          : std_logic;
signal sig_dir_up             : std_logic;
signal sig_dir_down           : std_logic;


begin
   RD_WR_MMU <= '0'; -- every operations with memmory are reading
   
   -- first value is actual position, next is new positions
   input_mux : process (CLK, X_WB, Y_WB, sig_X_new, sig_Y_new, sig_xy_sel)
   begin
      case sig_xy_sel is
         when '0'    => sig_X_in <= X_WB;
                        sig_Y_in <= Y_WB;
         when '1'    => sig_X_in <= sig_X_new; 
                        sig_Y_in <= sig_Y_new;
         when others => sig_X_in <= (others => '0');
                        sig_Y_in <= (others => '0');  
      end case; 
   end process;
     
   -- write to output registers
   output_registers : process (CLK, RST, sig_xy_we, sig_X_in, sig_Y_in)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_X <= (others => '0');
            sig_Y <= (others => '0');
         elsif (sig_xy_we = '1') then
            sig_X <= sig_X_in;
            sig_Y <= sig_Y_in;
         end if;
      end if;
   end process;
   
   -- connect to ECU outputs
   X_ECU        <= sig_X;
   Y_ECU        <= sig_Y;
   -- connect to MMU outputs
   X_MMU        <= sig_X;
   Y_MMU        <= sig_Y;
   
   -- write to save registers
   save_registers : process (CLK, RST, sig_xy_old_we, sig_X, sig_Y)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_X_old <= (others => '0');
            sig_Y_old <= (others => '0');
         elsif (sig_xy_old_we = '1') then
            sig_X_old <= sig_X;
            sig_Y_old <= sig_Y;
         end if;
      end if;
   end process;
   
   -- change x and y by direction
   output_mux : process (CLK, sig_X_old, sig_Y_old, sig_dir)
   begin
      case sig_dir is
         when "00"   => sig_X_new <= sig_X_old + 1;
                        sig_Y_new <= sig_Y_old;
         when "01"   => sig_X_new <= sig_X_old;
                        sig_Y_new <= sig_Y_old + 1;
         when "10"   => sig_X_new <= sig_X_old - 1;
                        sig_Y_new <= sig_Y_old;
         when "11"   => sig_X_new <= sig_X_old;
                        sig_Y_new <= sig_Y_old - 1;
         when others => sig_X_new <= (others => '0');
                        sig_Y_new <= (others => '0');  
      end case; 
   end process;
   
   -- write to save registers
   direction_registers : process (CLK, RST, sig_dir_up, sig_dir_down, sig_dir)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            sig_dir <= "00";
         elsif (sig_dir_up = '1') then
            --sig_dir <= sig_dir + 1;
            sig_dir <= sig_dir - 1;
         elsif (sig_dir_down = '1') then
            --sig_dir <= sig_dir - 1;
            sig_dir <= sig_dir + 1;
         end if;
      end if;
   end process;

-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic : process (state_reg, VLD_WB, VLD_MMU, VLD_ECU, DATA_IN_MMU, sig_X, sig_Y, RDY )
   begin
     state_next <= state_reg;  
     
     case state_reg is
        when init_state => 
		  if (RDY = '1') then
             state_next <= wait_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '0';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
		  else
             state_next <= init_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '0';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
		  end if;
        when wait_state => 
          state_next <= read_pos_state;
          ACT_WB        <= '0';
          sig_xy_sel    <= '0';
          sig_xy_we     <= '0';
          sig_xy_old_we <= '0';
          ACT_MMU       <= '0';
          sig_dir_up    <= '0';
          sig_dir_down  <= '0';
          ACT_ECU       <= '0';
        when read_pos_state =>
          if (VLD_WB = '1') then 
             state_next    <= add_first_state;
             ACT_WB        <= '1';
             sig_xy_sel    <= '0';
             sig_xy_we     <= '1';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          else 
             state_next <= read_pos_state;
             ACT_WB        <= '1';
             sig_xy_sel    <= '0';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          end if;                
        
       when add_first_state => 
         if (VLD_ECU = '1') then
             state_next <= goal_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '0';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '1';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '1';
          else
             state_next <= add_first_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '0';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '1';
          end if;
      
      when goal_state => 
         if (X_TARGET = sig_X and Y_TARGET = sig_Y) then
             state_next <= goal_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          else
             state_next <= load_out_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '1';
             ACT_ECU       <= '0';
          end if;
          
       when load_out_state => 
          state_next <= read_mem_state;
          ACT_WB        <= '0';
          sig_xy_sel    <= '1';
          sig_xy_we     <= '1';
          sig_xy_old_we <= '0';
          ACT_MMU       <= '0';
          sig_dir_up    <= '0';
          sig_dir_down  <= '0';
          ACT_ECU       <= '0';
          
      when read_mem_state => 
         if (VLD_MMU = '1') then
             state_next <= detect_bar_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '1';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          else
             state_next <= read_mem_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '1';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          end if;
          
          
      when detect_bar_state => 
         if (DATA_IN_MMU(1) = '0') then
             state_next <= read_mem_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          elsif (DATA_IN_MMU(0) = '1') then
             state_next <= load_out_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '1';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          else
             state_next <= go_to_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '0';
          end if;
       
       when go_to_state => 
         if (VLD_ECU = '1') then
             state_next <= goal_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '1';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '1';
          else
             state_next <= go_to_state;
             ACT_WB        <= '0';
             sig_xy_sel    <= '1';
             sig_xy_we     <= '0';
             sig_xy_old_we <= '0';
             ACT_MMU       <= '0';
             sig_dir_up    <= '0';
             sig_dir_down  <= '0';
             ACT_ECU       <= '1';
          end if;   
          
        when others => 
          state_next <= init_state;
          ACT_WB        <= '0';
          sig_xy_sel    <= '1';
          sig_xy_we     <= '0';
          sig_xy_old_we <= '0';
          ACT_MMU       <= '0';
          sig_dir_up    <= '0';
          sig_dir_down  <= '0';
          ACT_ECU       <= '0';
      end case;      
   end process;  
end architecture;
