-----------------------------------------------------------------------------
-- Project Name: Robot unit
-- File Name:    top_level.vhd
-- Description:  Masters and slave interfacec connect with Wishbone bus to testing
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         9.11.2012 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity TOP_LEVEL is
   generic
   (
      DATA_WIDTH     : integer := 16;  
      ADDR_WIDTH     : integer := 8;
      OFFSET_WIDTH   : integer := 4       
   );
   port(
      CLK: in std_logic;
      RST: in std_logic;
      
      -- SIF_WRITE_MASTER
      SIF_ACT           : in std_logic;
      SIF_VLD           : out std_logic;
      SIF_A             : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      SIF_B             : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      SIF_C             : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      SIF_S_0           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      SIF_S_1           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      SIF_S_2           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      SIF_S_3           : in std_logic_vector(DATA_WIDTH-1 downto 0);
      
      -- MU_READ_MASTER
      MU_ACT            : in std_logic;
      MU_VLD            : out std_logic;
      MU_X              : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      MU_Y              : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      MU_BAR            : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      
      -- PFU_READ_MASTER
      PFU_ACT           : in std_logic;
      PFU_VLD           : out std_logic;
      PFU_X             : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      PFU_Y             : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      
      -- PEU_SLAVE
      PEU_A             : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      PEU_B             : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      PEU_C             : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      PEU_X             : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      PEU_Y             : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      PEU_NOT_CYC_WRITE : out std_logic;
      PEU_NOT_CYC_READ  : out std_logic;
      
      -- BDU_SLAVE
      BDU_S_0           : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      BDU_S_1           : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      BDU_S_2           : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      BDU_S_3           : out std_logic_vector( DATA_WIDTH-1 downto 0 );
      BDU_BAR           : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      BDU_NOT_CYC_WRITE : out std_logic;
      BDU_NOT_CYC_READ  : out std_logic
   );
end entity TOP_LEVEL;

architecture ARCH_TOP_LEVEL of TOP_LEVEL is
  signal sif_dat_i : std_logic_vector(15 downto 0);
  signal sif_ack_i : std_logic;
  signal sif_dat_o : std_logic_vector(15 downto 0);
  signal sif_we_o  : std_logic;
  signal sif_sel_o : std_logic_vector(1 downto 0);
  signal sif_adr_o : std_logic_vector(7 downto 0);
  signal sif_cyc_o : std_logic;
  signal sif_stb_o : std_logic;
  
  signal mu_dat_i : std_logic_vector(15 downto 0);
  signal mu_ack_i : std_logic;
  signal mu_dat_o : std_logic_vector(15 downto 0);
  signal mu_we_o  : std_logic;
  signal mu_sel_o : std_logic_vector(1 downto 0);
  signal mu_adr_o : std_logic_vector(7 downto 0);
  signal mu_cyc_o : std_logic;
  signal mu_stb_o : std_logic;

  signal pfu_dat_i : std_logic_vector(15 downto 0);
  signal pfu_ack_i : std_logic;
  signal pfu_dat_o : std_logic_vector(15 downto 0);
  signal pfu_we_o  : std_logic;
  signal pfu_sel_o : std_logic_vector(1 downto 0);
  signal pfu_adr_o : std_logic_vector(7 downto 0);
  signal pfu_cyc_o : std_logic;
  signal pfu_stb_o : std_logic;
  
  signal peu_dat_o : std_logic_vector(15 downto 0);
  signal peu_ack_o : std_logic;
  signal peu_dat_i : std_logic_vector(15 downto 0);
  signal peu_we_i  : std_logic;
  signal peu_sel_i : std_logic_vector(1 downto 0);
  signal peu_adr_i : std_logic_vector(3 downto 0);
  signal peu_cyc_i : std_logic;
  signal peu_stb_i : std_logic;
  
  signal bdu_dat_o : std_logic_vector(15 downto 0);
  signal bdu_ack_o : std_logic;
  signal bdu_dat_i : std_logic_vector(15 downto 0);
  signal bdu_we_i  : std_logic;
  signal bdu_sel_i : std_logic_vector(1 downto 0);
  signal bdu_adr_i : std_logic_vector(3 downto 0);
  signal bdu_cyc_i : std_logic;
  signal bdu_stb_i : std_logic;
  
  begin

  intercon: entity work.intercon   
  port map(
      -- wishbone master port
      sif_dat_i => sif_dat_i,
      sif_ack_i => sif_ack_i,
      sif_dat_o => sif_dat_o,
      sif_we_o  => sif_we_o,
      sif_sel_o => sif_sel_o,
      sif_adr_o => sif_adr_o,
      sif_cyc_o => sif_cyc_o,
      sif_stb_o => sif_stb_o,
      
      mu_dat_i => mu_dat_i,
      mu_ack_i => mu_ack_i,
      mu_dat_o => mu_dat_o,
      mu_we_o  => mu_we_o,
      mu_sel_o => mu_sel_o,
      mu_adr_o => mu_adr_o,
      mu_cyc_o => mu_cyc_o,
      mu_stb_o => mu_stb_o,
      
      pfu_dat_i => pfu_dat_i,
      pfu_ack_i => pfu_ack_i,
      pfu_dat_o => pfu_dat_o,
      pfu_we_o  => pfu_we_o,
      pfu_sel_o => pfu_sel_o,
      pfu_adr_o => pfu_adr_o,
      pfu_cyc_o => pfu_cyc_o,
      pfu_stb_o => pfu_stb_o,
      -- wishbone slave port
      peu_dat_o => peu_dat_o,
      peu_ack_o => peu_ack_o,
      peu_dat_i => peu_dat_i,
      peu_we_i  => peu_we_i,
      peu_sel_i => peu_sel_i,
      peu_adr_i => peu_adr_i,
      peu_cyc_i => peu_cyc_i,
      peu_stb_i => peu_stb_i,
      
      bdu_dat_o => bdu_dat_o,
      bdu_ack_o => bdu_ack_o,
      bdu_dat_i => bdu_dat_i,
      bdu_we_i  => bdu_we_i,
      bdu_sel_i => bdu_sel_i,
      bdu_adr_i => bdu_adr_i,
      bdu_cyc_i => bdu_cyc_i,
      bdu_stb_i => bdu_stb_i,
      -- clock and reset
      clk            => CLK,
      reset          => RST

  ); 
  
  sif_wb : entity work.wb_sif_write_master_ent  
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_I          => sif_ack_i,
      DAT_I          => sif_dat_i,
      DAT_O          => sif_dat_o,
      STB_O          => sif_stb_o,
      ADR_O          => sif_adr_o,
      CYC_O          => sif_cyc_o,
      WE_O           => sif_we_o,
      -- Output port (non-WISHBONE signals)
      ACT            => SIF_ACT,
      VLD            => SIF_VLD,
      A              => SIF_A,
      B              => SIF_B,
      C              => SIF_C,
      S_0            => SIF_S_0,
      S_1            => SIF_S_1,
      S_2            => SIF_S_2,
      S_3            => SIF_S_3
  );
  
  mu_wb : entity work.wb_mu_read_master_ent  
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_I          => mu_ack_i,
      DAT_I          => mu_dat_i,
      DAT_O          => mu_dat_o,
      STB_O          => mu_stb_o,
      ADR_O          => mu_adr_o,
      CYC_O          => mu_cyc_o,
      WE_O           => mu_we_o,
      -- Output port (non-WISHBONE signals)
      ACT            => MU_ACT,
      VLD            => MU_VLD,
      X              => MU_X,
      Y              => MU_Y,
      BAR            => MU_BAR
  );
  
  pfu_wb : entity work.wb_pfu_read_master_ent  
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_I          => pfu_ack_i,
      DAT_I          => pfu_dat_i,
      DAT_O          => pfu_dat_o,
      STB_O          => pfu_stb_o,
      ADR_O          => pfu_adr_o,
      CYC_O          => pfu_cyc_o,
      WE_O           => pfu_we_o,
      -- Output port (non-WISHBONE signals)
      ACT            => PFU_ACT,
      VLD            => PFU_VLD,
      X              => PFU_X,
      Y              => PFU_Y
  );
  
  peu_wb : entity work.wb_peu_slave_ent   
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_O          => peu_ack_o,
      DAT_I          => peu_dat_i,
      DAT_O          => peu_dat_o,
      STB_I          => peu_stb_i,
      ADR_I          => peu_adr_i,
      CYC_I          => peu_cyc_i,
      WE_I           => peu_we_i,
      -- Output port (non-WISHBONE signals)
      A              => PEU_A,
      B              => PEU_B,
      C              => PEU_C,
      X              => PEU_X,
      Y              => PEU_Y,
      NOT_CYC_WRITE  => PEU_NOT_CYC_WRITE
      --NOT_CYC_READ   => PEU_NOT_CYC_READ
  );
  
  bdu_wb : entity work.wb_bdu_slave_ent   
  port map(
      CLK            => CLK,
      RST            => RST,
      -- Wishbone signals
      ACK_O          => bdu_ack_o,
      DAT_I          => bdu_dat_i,
      DAT_O          => bdu_dat_o,
      STB_I          => bdu_stb_i,
      ADR_I          => bdu_adr_i,
      CYC_I          => bdu_cyc_i,
      WE_I           => bdu_we_i,
      -- Output port (non-WISHBONE signals):
      S_0            => BDU_S_0,
      S_1            => BDU_S_1,
      S_2            => BDU_S_2,
      S_3            => BDU_S_3,
      BAR            => BDU_BAR,
      NOT_CYC_WRITE  => BDU_NOT_CYC_WRITE
      --NOT_CYC_READ   => BDU_NOT_CYC_READ
  );
end architecture ARCH_TOP_LEVEL;