-----------------------------------------------------------------------------
-- Project Name: Robot Unit
-- File Name:    wb_read_master.vhd
-- Description:  Read data from slave's registers specified by adress
-- Author:       Jakub Podivinsky <xpodiv01@stud.fit.vutbr.cz> 
-- Date:         25.2.2013 
-- --------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

-- ==========================================================================
--                              ENTITY DECLARATION
-- ==========================================================================
entity WB_MU_READ_MASTER_ENT is
   generic
   (
      DATA_WIDTH     : integer := 16;  
      ADDR_WIDTH     : integer := 8;
      OFFSET_WIDTH   : integer := 4       
   );
   
   port
   (
      CLK            : in std_logic;
      RST            : in std_logic;
      ACT            : in std_logic;
      VLD            : out std_logic;

      -- ----------------- WISHBONE MASTER interface -------------------------
      ACK_I          : in std_logic;
      DAT_I          : in std_logic_vector( DATA_WIDTH-1 downto 0 );
      DAT_O          : out std_logic_vector( DATA_WIDTH-1 downto 0 ) := (others => '0');
      STB_O          : out std_logic;
      ADR_O          : out std_logic_vector( ADDR_WIDTH-1 downto 0 );
      CYC_O          : out std_logic;
      WE_O           : out std_logic; 
     
      -- ----------------- DATA and ADRESS -----------------------------------      
      X              : out std_logic_vector(DATA_WIDTH-1 downto 0);
      Y              : out std_logic_vector(DATA_WIDTH-1 downto 0);
      BAR            : out std_logic_vector(DATA_WIDTH-1 downto 0)
   );
   
end entity;

-- ==========================================================================
--                           ARCHITECTURE DESCRIPTION
-- ==========================================================================
architecture WB_MU_READ_MASTER_ARCH of WB_MU_READ_MASTER_ENT is

-- ==========================================================================
--                                      TYPES
-- ==========================================================================
type state_type is (init_state,first_edge_state, second_edge_state, wait_state, handshake_state);

-- ==========================================================================
--                                    CONSTANTS
-- ==========================================================================
constant const_in_num     : std_logic_vector(3 downto 0) := "0011"; -- number of writing values
constant const_addr_X     : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00010011"; --adress of register A
constant const_addr_Y     : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00010101"; --adress of register B
constant const_addr_BAR   : std_logic_vector(ADDR_WIDTH-1 downto 0):= "00100101"; --adress of register B
-- ==========================================================================
--                                     SIGNALS
-- ==========================================================================
-- FSM signals 
signal state_reg, state_next : state_type;

signal sig_addr   : std_logic_vector( ADDR_WIDTH-1 downto 0);--data address

signal n          : std_logic_vector( 3 downto 0 );--number of writed values
signal sig_go     : std_logic; --signal to increment counter of writed values
signal sig_reset  : std_logic; --signal to reset counter of writed values

signal sig_we     : std_logic; --write enable to output registers 

--output registers
signal reg_X            : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_Y            : std_logic_vector(DATA_WIDTH-1 downto 0);
signal reg_BAR          : std_logic_vector(DATA_WIDTH-1 downto 0);

begin
-- ------------------------- FSM --------------------------------------------   
   -- state logic
   fsm_state_logic : process (CLK)
   begin
     if (rising_edge(CLK)) then
        if (RST = '1') then
          state_reg <= init_state;
        else
          state_reg <= state_next;
        end if;   
     end if;   
   end process;
   
   -- next state logic
   fsm_next_state_logic: process(state_reg, ACT, ACK_I, n) --stavova logika
   begin
      state_next <= state_reg;
      case state_reg is
         when init_state =>
            if (ACT = '1') then
               sig_go <= '0';
               sig_reset <= '1';
               VLD <= '0';
               state_next <= first_edge_state;
            else
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= init_state;
            end if;
         when first_edge_state => 
            sig_go <= '1';
            sig_reset <= '0';
            VLD <= '0';
            state_next <= second_edge_state;   
         when second_edge_state =>
            if (ACK_I = '1' and n = const_in_num) then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= handshake_state;
            elsif (ACK_I = '1') then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= wait_state;
            else
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= second_edge_state;
            end if;
         when wait_state =>
            if (n = const_in_num) then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= init_state;
            else
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '0';
               state_next <= first_edge_state;
            end if;
         when handshake_state =>
            if (ACT = '1') then
               sig_go <= '0';
               sig_reset <= '0';
               VLD <= '1';
               state_next <= handshake_state; 
            else
               sig_go <= '0';
               sig_reset <= '1';
               VLD <= '0';
              state_next <= init_state;
         end if;
      end case;
   end process;  
   
-- Moore output logic
   moore_output_logic: process(state_reg, sig_addr)
   begin  
      case state_reg is
         when init_state => 
            STB_O <= '0';
            CYC_O <= '0';
            WE_O <= '0';
            sig_we <= '0';
         when first_edge_state =>
            STB_O <= '1';
            CYC_O <= '1';
            WE_O <= '0';
            sig_we <= '0';
         when second_edge_state =>  
            STB_O <= '1';
            CYC_O <= '1';
            WE_O <= '0';  
            sig_we <= '1';
         when wait_state =>  
            STB_O <= '0';
            CYC_O <= '1';
            WE_O <= '0';     
            sig_we <= '0';
         when handshake_state => 
            STB_O <= '0';
            CYC_O <= '0';
            WE_O <= '0';
            sig_we <= '0';
         when others =>
            CYC_O <= '0';
            STB_O <= '0';
            WE_O <= '0';
            sig_we <= '0';
      end case;
   end process;
   
   -- counter to count number of reading values
   counter : process (sig_go, sig_reset, CLK)
   variable COUNT_INT: integer;
   begin
      if( rising_edge( CLK ) ) then
         if (sig_reset = '1') then
            COUNT_INT := 0;
         elsif (sig_go='1') then
            if COUNT_INT = 15 then
               COUNT_INT := 0;
            else
               COUNT_INT := COUNT_INT + 1; 
            end if;
         end if; 
      end if;
      n <= conv_std_logic_vector(COUNT_INT, 4);
   end process;

   -- set adress of reading registers
   mux : process(n)
   begin
      case n is
         when "0001"   =>  sig_addr <= const_addr_X;
         when "0010"   =>  sig_addr <= const_addr_Y;
         when "0011"   =>  sig_addr <= const_addr_BAR;
         when others =>    sig_addr <= (others => '0');   
      end case;  
   end process;
   
   ADR_O <= sig_addr;
   
   -- output registers to store receive values
   write_registers : process (CLK, RST, n)
   begin
      if( rising_edge( CLK ) ) then
         if (RST = '1') then
            reg_X <= (others => '0');
            reg_Y <= (others => '0');
            reg_BAR <= (others => '0');
         elsif (sig_we = '1') then
            case n is
               when "0001" => reg_X    <= DAT_I;
               when "0010" => reg_Y    <= DAT_I;
               when "0011" => reg_BAR  <= DAT_I;
               when others   => null;
            end case;
         end if;
      end if;
   end process;
		
	X <= reg_X;
   Y <= reg_Y;
   BAR <= reg_BAR;

end architecture;
